#include <algorithm>
#include <cmath>
#include <cstdlib>
#include <glm/geometric.hpp>
#include <iostream>
#include <map>
#include <memory>
#include <tuple>
#include <vector>

#include "BulletCollision/CollisionDispatch/btCollisionObject.h"
#include "BulletCollision/CollisionShapes/btBoxShape.h"
#include "LinearMath/btVector3.h"
#include "constants.h"
#include "elements.h"
#include "engine.h"
#include "gameobject.h"
#include "input.h"
#include "internal/physics.h"
#include "kekengine.h"
#include "light.h"
#include "mesh.h"
#include "player.h"
#include "scene.h"
#include "texture.h"
#include "ui.h"
#include "uielements.h"
#include "utils.h"

using namespace kek;

#define CHUNK_SIZE 64
#define CHUNK_RADIUS 2
#define WALLS_PER_CHUNK 20
#define WALL_HEIGHT 10
#define LIGHT_SPACING 16

static ButtonElement *buttonPlay;

void gameLoop(GLFWwindow *window, void *) {}

void startGame() {
	ConsoleElement *console = new ConsoleElement(uiPx(10), uiPx(10), uiSw(1) - uiPx(20), uiSh(1) - uiPx(20));
	console->onCommand = CommandCallback([](CommandEvent event, void *data) {
		ConsoleElement *_console = (ConsoleElement *) data;
		_console->textElement->setText(_console->textElement->getText() + "\n" + "> " + event.command);
		_console->textFieldElement->setText("");
		UI::focusElement(_console->textFieldElement);
	},
		console);
	UI::addElement(console);
}

int main() {
	if(Engine::init() != KEK_SUCCESS)
		return 1;

	buttonPlay = new ButtonElement(uiSw(0.5), uiSh(0.5), uiPx(500), uiPx(75));
	buttonPlay->origin = Origin::CENTER;
	buttonPlay->text->setText("Play");
	buttonPlay->text->sizePixels = 50;
	buttonPlay->text->color = Colors::BLACK;
	buttonPlay->onClick = Callable(startGame);

	UI::addElement(buttonPlay);
	Input::setCursorMode(GLFWCursorMode::FREE);

	if(Engine::start() != KEK_SUCCESS)
		return 1;

	Engine::exit();
}
